--[[---------------------------------------------------------------------------------------------------
											EmbedHandler-1.0
											
Description:
Author: Copyright 2008 João Cardoso (Jaliborc)
Dependencies: LibStub
License: GNU General Public License - Version 3 (<http://www.gnu.org/licenses/> for more information)
--]]------------------------------------------------------------------------------------------------

local lib, old = LibStub:NewLibrary('EmbedHandler-1.0', 2)
if not lib then
	return
end

local type, pairs, unpack, select = type, pairs, unpack, select --speed up!
local empty, data, isFunc = {}

local function Embed(self, target, data)
	for k,v in pairs(self) do
		isFunc = type(v) == 'function'
		if (isFunc and not data[k]) or (data[k] and not isFunc) then
			target[k] = self[k]
		end
	end
end

local function SpecificEmbed(self, target, data)
	for i,k in ipairs(data) do
		target[k] = self[k]
	end
end

local function SaveData(self, target, data)
	lib.embeds[self] = lib.embeds[self] or {}
	lib.embeds[self][target] = data
end


--[[ API ]]--

function lib:Embed(target, ...)
	data = {}
	for i = 1, select('#', ...) do
		data[select(i, ...)] = true
	end
	
	Embed(self, target, data)
	SaveData(self, target, data)
end

function lib:SpecificEmbed(target, ...)
	data = {specific = 1, ...}
	
	SpecificEmbed(self, target, data)
	SaveData(self, target, data)
end

function lib:IterateEmbeds()
	return pairs(lib.embeds[self] or empty)
end
	
function lib:UpdateEmbeds()
	for target,data in lib.IterateEmbeds(self) do --it's safer to index the lib
		if data.specific ~= 1 then
			Embed(self, target, data)
		else
			SpecificEmbed(self, target, data)
		end
	end
end


--[[ Update Embeded Objects ]]--

if old then
	lib:UpdateEmbeds()
else
	lib.embeds = {}
end